@@ -378,6 +378,7 @@ g_vfs_backend_mtp_init (GVfsBackendMtp *backend)
 {
   g_debug ("(I) g_vfs_backend_mtp_init\n");
   GMountSpec *mount_spec;
+  const char *debug;
 
   g_mutex_init (&backend->mutex);
   g_vfs_backend_set_display_name (G_VFS_BACKEND (backend), "mtp");
@@ -394,6 +395,22 @@ g_vfs_backend_mtp_init (GVfsBackendMtp *backend)
                                            backend, 1, FALSE, NULL);
 #endif
 
+  debug = g_getenv ("GVFS_MTP_DEBUG");
+  if (debug != NULL) {
+    int level;
+
+    if (g_ascii_strcasecmp ("ptp", debug) == 0)
+      level = LIBMTP_DEBUG_PTP;
+    else if (g_ascii_strcasecmp ("usb", debug) == 0)
+      level = LIBMTP_DEBUG_USB | LIBMTP_DEBUG_PTP;
+    else if (g_ascii_strcasecmp ("data", debug) == 0)
+      level = LIBMTP_DEBUG_DATA | LIBMTP_DEBUG_USB | LIBMTP_DEBUG_PTP;
+    else /* "all" */
+      level = LIBMTP_DEBUG_ALL;
+
+    LIBMTP_Set_Debug (level);
+  }
+
   g_debug ("(I) g_vfs_backend_mtp_init done.\n");
 }
 
@@ -914,14 +931,14 @@ do_mount (GVfsBackend *backend,
   op_backend->volume_symbolic_icon = g_vfs_get_volume_symbolic_icon (device);
   g_object_unref (device);
 
+  g_signal_connect (op_backend->gudev_client, "uevent", G_CALLBACK (on_uevent), op_backend);
+
+  op_backend->file_cache = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+
   LIBMTP_Init ();
 
   get_device (backend, host, G_VFS_JOB (job));
   if (!G_VFS_JOB (job)->failed) {
-    g_signal_connect (op_backend->gudev_client, "uevent", G_CALLBACK (on_uevent), op_backend);
-
-    op_backend->file_cache = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
-
     GMountSpec *mtp_mount_spec = g_mount_spec_new ("mtp");
     g_mount_spec_set (mtp_mount_spec, "host", host);
     g_vfs_backend_set_mount_spec (backend, mtp_mount_spec);
